import { create } from 'zustand'
import { ListConversationModel } from '@/feature/messenger/models/ListConversationModel'
import { getListConversations } from '@/feature/messenger/services/chat/ChatService'
import NotificationAudio from '@/feature/messenger/utils/NotificationAudio'
import useConfigChatStore from '@/feature/messenger/store/chat_store/useConfigChatStore'

type typeFilterSearch = {
    type: string,
    value: string,
    tags: string[]
}
type TypeActionsConversations = {
    setListConversation: (conversation: ListConversationModel[]) => void
    getListConversation: (id_chatbot: string) => void,
    setSelectedConversation: (conversation: ListConversationModel) => void
    updateConversation: (conversation: ListConversationModel) => void
    removeConversation: (conversation: ListConversationModel) => void,
    searchConversations: (filter: typeFilterSearch) => void
    resetSelectedConversation: () => void
    addNewConversation: (conversation: ListConversationModel) => void,
    setFilter: (filter: typeFilterSearch) => void,
    applyFilters: () => void,
    removeFilter: (filter: typeFilterSearch) => void
    updateTags: (conversation: ListConversationModel, tags: any[]) => void
    validateNotifications: (defaultList: ListConversationModel[], newConversations: ListConversationModel[]) => void
}
type TypeConversations = {
    listConversation: ListConversationModel[],
    defaultListConversation: ListConversationModel[],
    selectedConversation: ListConversationModel,
    filter: typeFilterSearch,
    filters: typeFilterSearch[],
    actions: TypeActionsConversations
}

const getSoundPermissions = () => {
    const permisions = useConfigChatStore.getState().permisions;
    return permisions.sonido
}
const validateNotificationConversation = (defaultListConversation: ListConversationModel[], previousConversations: ListConversationModel[]) => {
    // Crea una instancia de AudioContext    
    if (previousConversations.length == 0) {
        return
    }

    let contadorSound: number = 0;
    defaultListConversation.forEach((conversation) => {
        //conversation.tags = [];
        conversation.sinLeer = Number(conversation.sinLeer);
    });

    previousConversations.forEach((conversation) => {
        //conversation.tags = [];
        conversation.sinLeer = Number(conversation.sinLeer);
    });

    if (defaultListConversation.length > previousConversations.length) {
        contadorSound += 1;
    }    

    if (defaultListConversation.length === previousConversations.length) {
        const json1 = JSON.stringify(defaultListConversation);
        const json2 = JSON.stringify(previousConversations);
        if (!(json1 === json2)) {
            contadorSound += 1;
        }
    }
    if (contadorSound > 0) {
        const sonido = getSoundPermissions();
        const { playNotificationSound } = NotificationAudio(sonido);
        playNotificationSound();
    }
}
const orderConversations = (conversations: ListConversationModel[]) => {
    return conversations.sort((a, b) => {
        if (a.fecha < b.fecha) return 1;
        if (a.fecha > b.fecha) return -1;
        return 0;
    })
}
const useConversationsStore = create<TypeConversations>((set, get) => ({
    listConversation: [],
    defaultListConversation: [],
    filter: { type: 'sesion', value: '', tags: [] },
    selectedConversation: {
        sesion: '',
        nombre: '',
        rsocial: '',
        conversacion: '',
        tipomensaje: '',
        sinLeer: 0,
        alerta_demora: '',
        fecha: '',
        tags: []
    },
    filters: [],
    actions: {
        setListConversation: (conversation: ListConversationModel[]) => set({ listConversation: conversation, defaultListConversation: conversation }),
        getListConversation: (id_chatbot: string) => {
            if (id_chatbot === '') {
                return
            }
            try {
                getListConversations(id_chatbot).then((res) => {
                    if (res.status == 500) {
                        return
                    } else {
                        if (res.data.length > 0) {
                            const defaultList = get().defaultListConversation;
                            const newConversations = res.data;
                            /*const updatedConversations = res.data.map((newConv: ListConversationModel) => {
                                const existingConv = defaultListConversation.find(conv => conv.sesion === newConv.sesion);
                                return existingConv ? { ...newConv, tags: existingConv.tags } : newConv;
                            });*/
                            set({ defaultListConversation: orderConversations(newConversations) })
                            get().actions.applyFilters();
                            validateNotificationConversation(defaultList, newConversations)
                        }
                    }
                })
            } catch (error) {

            }
        },
        validateNotifications: (defaultList: ListConversationModel[], newConversations: ListConversationModel[]) => {
            validateNotificationConversation(defaultList, newConversations);
        },
        setSelectedConversation: (conversation: ListConversationModel) => {
            set({ selectedConversation: conversation })
            set({ listConversation: get().listConversation.map((item) => item.sesion === conversation.sesion ? conversation : item) })
            //set({ defaultListConversation: get().defaultListConversation.map((item) => item.sesion === conversation.sesion ? conversation : item) })
            get().actions.applyFilters();
        },
        updateConversation: (conversation: ListConversationModel) => {
            set((state) => ({
                listConversation: state.listConversation.map((item) => {
                    if (item.sesion === conversation.sesion) {
                        return conversation
                    }
                    return item
                })
            }))
            set({ defaultListConversation: get().defaultListConversation.map((item) => {
                if (item.sesion === conversation.sesion) {
                        return conversation
                    }
                    return item
                })
            })
            set({ listConversation: orderConversations(get().listConversation) })
            set({ defaultListConversation: orderConversations(get().defaultListConversation) })
        },
        removeConversation: (conversation: ListConversationModel) => {
            set((state) => ({
                listConversation: state.listConversation.filter((item) => item.sesion !== conversation.sesion)
            }))
            set((state) => ({
                defaultListConversation: state.defaultListConversation.filter((item) => item.sesion !== conversation.sesion)
            }))
            set({ selectedConversation: { sesion: '', nombre: '', rsocial: '', conversacion: '', tipomensaje: '', sinLeer: 0, alerta_demora: '', fecha: '', tags: [] } })
        },
        searchConversations: (filter: typeFilterSearch) => {
            set({ filter: filter })
            if (filter.type === 'sesion') {
                const filteredConversations = get().defaultListConversation.filter(item => {
                    const nameComplete = `${item.nombre.toLowerCase()} ${item.sesion.toLowerCase()}`;
                    return nameComplete.includes(filter.value.toLowerCase());
                });
                const newlistConversation = filter.value ? filteredConversations : get().defaultListConversation;
                set({ listConversation: newlistConversation })
            }
        },
        resetSelectedConversation: () => {
            set({ selectedConversation: { sesion: '', nombre: '', rsocial: '', conversacion: '', tipomensaje: '', sinLeer: 0, alerta_demora: '', fecha: '', tags: [] } })
        },
        addNewConversation: (conversation: ListConversationModel) => {
            set({ listConversation: [{...conversation}, ...get().listConversation] })
            set({ defaultListConversation: [{...conversation}, ...get().defaultListConversation] })
        },
        setFilter: (newFilter: typeFilterSearch) => {
            set((state) => {
                const existingFilterIndex = state.filters.findIndex(f => f.type === newFilter.type);
                if (existingFilterIndex !== -1) {
                    // Si el filtro ya existe, lo actualizamos
                    const updatedFilters = [...state.filters];
                    updatedFilters[existingFilterIndex] = newFilter;
                    return { filters: updatedFilters };
                } else {
                    // Si el filtro no existe, lo añadimos
                    return { filters: [...state.filters, newFilter] };
                }
            });
            get().actions.applyFilters();
        },
        applyFilters: () => {
            const { defaultListConversation, filters } = get();
            let filtered = defaultListConversation;

            filters.forEach(filter => {
                switch (filter.type) {
                    case 'search':
                        if (filter.value) {
                            filtered = filtered.filter(item => {
                                const nameComplete = `${item.nombre.toLowerCase()} ${item.sesion.toLowerCase()}`;
                                return nameComplete.includes(filter.value.toLowerCase());
                            });
                            filtered = filter.value != '' ? filtered : get().defaultListConversation;
                        }
                        break;
                    case 'unread':
                        if (filter.value === 'unread') {
                            filtered = filtered.filter(conv => conv.sinLeer > 0);
                        }
                        break;
                    case 'tags':
                        if (filter.tags.length > 0) {
                            filtered = filtered.filter((conv) => {
                                return filter.tags.some(tag => conv.tags.some((tagConv: { id: number }) => tagConv.id === Number(tag)));
                            });
                        }
                        break;
                    // Puedes añadir más casos aquí para nuevos tipos de filtros
                }
            });
            set({ listConversation: orderConversations(filtered) });
        },
        removeFilter: (filter: typeFilterSearch) => {
            set((state) => ({
                filters: state.filters.filter((item) => item.type !== filter.type)
            }))
            get().actions.applyFilters();
        },
        updateTags: (conversation: ListConversationModel, tags: any[]) => {
            set((state) => ({
                defaultListConversation: state.defaultListConversation.map((item) => {
                    if (item.sesion === conversation.sesion) {
                        return { ...item, tags: tags }
                    }
                    return item
                })
            }))
        }
    }
}))

export default useConversationsStore
